==> cramr_ui_lib-1.0.7.tar.gz: lib/settings/settings_controller.dart <==
import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'settings_service.dart';

/// A class that many Widgets can interact with to read user settings, update
/// user settings, or listen to user settings changes.
///
/// Controllers glue Data Services to Flutter Widgets. The SettingsController
/// uses the SettingsService to store and retrieve user settings.
class SettingsController with ChangeNotifier {
  SettingsController(
      this._settingsService, this._themeMode, this._sharedPreferences);

  // Make SettingsService a private variable so it is not used directly.
  final SettingsService _settingsService;

  // Make SharedPreferences private so it is not used directly
  final SharedPreferences _sharedPreferences;

  // Make ThemeMode a private variable so it is not updated directly without
  // also persisting the changes with the SettingsService.
  ThemeMode _themeMode;

  // Allow Widgets to read the user's preferred ThemeMode.
  ThemeMode get themeMode => _themeMode;

  // Make CardinalitySelection a private variable so that it is not updated
  // directly without persisting changes with the SettingsService
  late final CardinalitySelection _cardinalitySelection;

  // Allow Widgets to read the user's preferred CardinalitySelection
  CardinalitySelection get cardinalitySelection => _cardinalitySelection;

  /// Load the user's settings from the SettingsService. It may load from a
  /// local database or the internet. The controller only knows it can load the
  /// settings from the service.
  Future<void> loadSettings() async {
    _themeMode = _settingsService.themeMode();

    // _cardinalitySelection = await _settingsService.cardinalitySelection();
    _cardinalitySelection = CardinalitySelection(
        _sharedPreferences.getBool("cardinality.singularEnabled") ?? true,
        _sharedPreferences.getBool("cardinality.pluralEnabled") ?? true);

    // Important! Inform listeners a change has occurred.
    notifyListeners();
  }

  /// Update and persist the cardinality settings based on the user's selection.
  Future<void> updateCardinalitySelection() async {
    await _settingsService.updateCardinalitySelection(_cardinalitySelection);
  }

  /// Update and persist the ThemeMode based on the user's selection.
  Future<void> updateThemeMode(ThemeMode? newThemeMode) async {
    if (newThemeMode == null) return;

    // Dot not perform any work if new and old ThemeMode are identical
    //if (newThemeMode == _themeMode) return;

    // Otherwise, store the new theme mode in memory
    _themeMode = newThemeMode;

    // Important! Inform listeners a change has occurred.
    notifyListeners();

    // Persist the changes to a local database or the internet using the
    // SettingService.
    await _settingsService.updateThemeMode(newThemeMode);
  }
}

==> cramr_ui_lib-1.0.8.tar.gz: lib/settings/settings_controller.dart <==
import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/streak_engine.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'settings_service.dart';

/// A class that many Widgets can interact with to read user settings, update
/// user settings, or listen to user settings changes.
///
/// Controllers glue Data Services to Flutter Widgets. The SettingsController
/// uses the SettingsService to store and retrieve user settings.
class SettingsController with ChangeNotifier {
  SettingsController(this._settingsService, this._themeMode,
      this._sharedPreferences);

  // Make SettingsService a private variable so it is not used directly.
  final SettingsService _settingsService;

  // Make SharedPreferences private so it is not used directly
  final SharedPreferences _sharedPreferences;

  // Make ThemeMode a private variable so it is not updated directly without
  // also persisting the changes with the SettingsService.
  ThemeMode _themeMode;

  // Allow Widgets to read the user's preferred ThemeMode.
  ThemeMode get themeMode => _themeMode;

  // Make CardinalitySelection a private variable so that it is not updated
  // directly without persisting changes with the SettingsService
  late final CardinalitySelection _cardinalitySelection;

  // Allow Widgets to read the user's preferred CardinalitySelection
  CardinalitySelection get cardinalitySelection => _cardinalitySelection;

  // Make StreakEngine a private variable so that it is not updated
  // directly without persisting changes with the SettingsService
  late StreakEngine _streakEngine;

  // Allow Widgets to read the user's StreakEngine
  StreakEngine get streakEngine => _streakEngine;

  /// Load the user's settings from the SettingsService. It may load from a
  /// local database or the internet. The controller only knows it can load the
  /// settings from the service.
  Future<void> loadSettings() async {
    _themeMode = _settingsService.themeMode();

    // _cardinalitySelection = await _settingsService.cardinalitySelection();
    _cardinalitySelection = CardinalitySelection(
        _sharedPreferences.getBool("cardinality.singularEnabled") ?? true,
        _sharedPreferences.getBool("cardinality.pluralEnabled") ?? true);

    var engineVar = _sharedPreferences.get('streakEngine');
    if (engineVar == null) {
      _streakEngine = StreakEngine();
    } else {
      _streakEngine = engineVar as StreakEngine;
    }

    // Important! Inform listeners a change has occurred.
    notifyListeners();
  }

  /// Update and persist the cardinality settings based on the user's selection.
  Future<void> updateCardinalitySelection() async {
    await _settingsService.updateCardinalitySelection(_cardinalitySelection);
  }

  /// Update and persist the ThemeMode based on the user's selection.
  Future<void> updateThemeMode(ThemeMode? newThemeMode) async {
    if (newThemeMode == null) return;

    // Dot not perform any work if new and old ThemeMode are identical
    //if (newThemeMode == _themeMode) return;

    // Otherwise, store the new theme mode in memory
    _themeMode = newThemeMode;

    // Important! Inform listeners a change has occurred.
    notifyListeners();

    // Persist the changes to a local database or the internet using the
    // SettingService.
    await _settingsService.updateThemeMode(newThemeMode);
  }

  /// Update and persist the StreakEngine based on the user's actions.
  Future<void> updateStreakEngine(StreakEngine? updatedStreakEngine) async {
    if (updatedStreakEngine == null) return;

    //TODO refactor so that we use the controller as a business facade

    // Inform listeners
    notifyListeners();

    // Persist changes
    await _settingsService.updateStreakEngine(updatedStreakEngine);
  }
}
